package java.beans;

import java.util.EventListener;
import javax.swing.JMenuItem;
import javax.swing.event.ChangeListener;
import java.awt.event.ComponentListener;
import java.awt.Component;
import java.lang.reflect.Modifier;
import sun.reflect.misc.ReflectUtil;
import java.lang.reflect.Field;
import java.lang.reflect.Array;
import java.util.Objects;
import java.lang.reflect.Method;
import sun.reflect.misc.MethodUtil;

public class DefaultPersistenceDelegate extends PersistenceDelegate
{
    private static final String[] EMPTY;
    private final String[] constructor;
    private Boolean definesEquals;
    
    public DefaultPersistenceDelegate() {
        this.constructor = DefaultPersistenceDelegate.EMPTY;
    }
    
    public DefaultPersistenceDelegate(String[] array) {
        this.constructor = ((array == null) ? DefaultPersistenceDelegate.EMPTY : ((String[])array.clone()));
    }
    
    private static boolean definesEquals(Class<?> clazz) {
        try {
            return clazz == clazz.getMethod("equals", new Class[] { Object.class }).getDeclaringClass();
        }
        catch (NoSuchMethodException ex) {
            return false;
        }
    }
    
    private boolean definesEquals(Object o) {
        if (this.definesEquals != null) {
            return this.definesEquals == Boolean.TRUE;
        }
        int definesEquals = definesEquals(o.getClass());
        this.definesEquals = (definesEquals ? Boolean.TRUE : Boolean.FALSE);
        return definesEquals;
    }
    
    @Override
    protected boolean mutatesTo(Object o, Object obj) {
        return (this.constructor.length == 0 || !this.definesEquals(o)) ? super.mutatesTo(o, obj) : o.equals(obj);
    }
    
    @Override
    protected Expression instantiate(Object obj, Encoder encoder) {
        int length = this.constructor.length;
        Class<Object> class1 = obj.getClass();
        Object[] array = new Object[length];
        for (int i = 0; i < length; ++i) {
            try {
                array[i] = MethodUtil.invoke(this.findMethod(class1, this.constructor[i]), obj, new Object[0]);
            }
            catch (Exception ex) {
                encoder.getExceptionListener().exceptionThrown(ex);
            }
        }
        return new Expression(obj, obj.getClass(), "new", array);
    }
    
    private Method findMethod(Class<?> clazz, String s) {
        if (s == null) {
            throw new IllegalArgumentException("Property name is null");
        }
        PropertyDescriptor propertyDescriptor = getPropertyDescriptor(clazz, s);
        if (propertyDescriptor == null) {
            throw new IllegalStateException("Could not find property by the name " + s);
        }
        Method readMethod = propertyDescriptor.getReadMethod();
        if (readMethod == null) {
            throw new IllegalStateException("Could not find getter for the property " + s);
        }
        return readMethod;
    }
    
    private void doProperty(Class<?> clazz, PropertyDescriptor propertyDescriptor, Object o, Object o2, Encoder encoder) throws Exception {
        Method readMethod = propertyDescriptor.getReadMethod();
        Method writeMethod = propertyDescriptor.getWriteMethod();
        if (readMethod != null && writeMethod != null) {
            Expression expression = new Expression(o, readMethod.getName(), new Object[0]);
            Expression expression2 = new Expression(o2, readMethod.getName(), new Object[0]);
            Object value = expression.getValue();
            Object value2 = expression2.getValue();
            encoder.writeExpression(expression);
            if (!Objects.equals(value2, encoder.get(value))) {
                Object[] array = (Object[])(Object[])propertyDescriptor.getValue("enumerationValues");
                if (array instanceof Object[] && Array.getLength(array) % 3 == 0) {
                    Object[] array2 = (Object[])(Object[])array;
                    for (int i = 0; i < array2.length; i += 3) {
                        try {
                            Field field = clazz.getField((String)array2[i]);
                            if (field.get(null).equals(value)) {
                                encoder.remove(value);
                                encoder.writeExpression(new Expression(value, field, "get", new Object[] { null }));
                            }
                        }
                        catch (Exception ex) {}
                    }
                }
                invokeStatement(o, writeMethod.getName(), new Object[] { value }, encoder);
            }
        }
    }
    
    static void invokeStatement(Object o, String s, Object[] array, Encoder encoder) {
        encoder.writeStatement(new Statement(o, s, array));
    }
    
    private void initBean(Class<?> clazz, Object o, Object o2, Encoder encoder) {
        for (Field field : clazz.getFields()) {
            if (ReflectUtil.isPackageAccessible(field.getDeclaringClass())) {
                int modifiers = field.getModifiers();
                if (!Modifier.isFinal(modifiers) && !Modifier.isStatic(modifiers) && !Modifier.isTransient(modifiers)) {
                    try {
                        Expression o3 = new Expression(field, "get", new Object[] { o });
                        Expression o4 = new Expression(field, "get", new Object[] { o2 });
                        Object value = o3.getValue();
                        Object value2 = o4.getValue();
                        encoder.writeExpression(o3);
                        if (!Objects.equals(value2, encoder.get(value))) {
                            encoder.writeStatement(new Statement(field, "set", new Object[] { o, value }));
                        }
                    }
                    catch (Exception ex) {
                        encoder.getExceptionListener().exceptionThrown(ex);
                    }
                }
            }
        }
        BeanInfo beanInfo;
        try {
            beanInfo = Introspector.getBeanInfo(clazz);
        }
        catch (IntrospectionException ex2) {
            return;
        }
        for (PropertyDescriptor propertyDescriptor : beanInfo.getPropertyDescriptors()) {
            if (!propertyDescriptor.isTransient()) {
                try {
                    this.doProperty(clazz, propertyDescriptor, o, o2, encoder);
                }
                catch (Exception ex3) {
                    encoder.getExceptionListener().exceptionThrown(ex3);
                }
            }
        }
        if (!Component.class.isAssignableFrom(clazz)) {
            return;
        }
        for (EventSetDescriptor eventSetDescriptor : beanInfo.getEventSetDescriptors()) {
            if (!eventSetDescriptor.isTransient()) {
                Class<Object> listenerType = eventSetDescriptor.getListenerType();
                if (listenerType != ComponentListener.class && (listenerType != ChangeListener.class || clazz != JMenuItem.class)) {
                    EventListener[] array = new EventListener[0];
                    EventListener[] array2 = new EventListener[0];
                    EventListener[] array3;
                    EventListener[] array4;
                    try {
                        Method getListenerMethod = eventSetDescriptor.getGetListenerMethod();
                        array3 = (EventListener[])(EventListener[])MethodUtil.invoke(getListenerMethod, o, new Object[0]);
                        array4 = (EventListener[])(EventListener[])MethodUtil.invoke(getListenerMethod, o2, new Object[0]);
                    }
                    catch (Exception ex4) {
                        try {
                            Method method = clazz.getMethod("getListeners", new Class[] { Class.class });
                            array3 = (EventListener[])(EventListener[])MethodUtil.invoke(method, o, new Object[] { listenerType });
                            array4 = (EventListener[])(EventListener[])MethodUtil.invoke(method, o2, new Object[] { listenerType });
                        }
                        catch (Exception ex5) {
                            return;
                        }
                    }
                    String name = eventSetDescriptor.getAddListenerMethod().getName();
                    for (int l = array4.length; l < array3.length; ++l) {
                        invokeStatement(o, name, new Object[] { array3[l] }, encoder);
                    }
                    String name2 = eventSetDescriptor.getRemoveListenerMethod().getName();
                    for (int length4 = array3.length; length4 < array4.length; ++length4) {
                        invokeStatement(o, name2, new Object[] { array4[length4] }, encoder);
                    }
                }
            }
        }
        return;
    }
    
    @Override
    protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
        super.initialize(clazz, o, o2, encoder);
        if (o.getClass() == clazz) {
            this.initBean(clazz, o, o2, encoder);
        }
    }
    
    private static PropertyDescriptor getPropertyDescriptor(Class<?> clazz, String s) {
        try {
            PropertyDescriptor[] propertyDescriptors = Introspector.getBeanInfo(clazz).getPropertyDescriptors();
            int length = propertyDescriptors.length;
            int i = 0;
            while (i < length) {
                PropertyDescriptor propertyDescriptor = propertyDescriptors[i];
                if (s.equals(propertyDescriptor.getName())) {
                    return propertyDescriptor;
                }
                ++i;
            }
        }
        catch (IntrospectionException ex) {}
        return null;
    }
    
    static {
        EMPTY = new String[0];
    }
}
